"""OpenAI Codex provider backed by the ChatGPT Codex Responses endpoint."""

import base64
import io
import json
import wave
from importlib.util import find_spec
from typing import Any, AsyncGenerator, BinaryIO, Callable

if find_spec("openai") is None or find_spec("websockets") is None:
    raise RuntimeError(
        "OpenAI Codex provider requires the codex extra. "
        "Install with `pip install 'aceai[codex]'`."
    )

from openai import AsyncOpenAI
from websockets.asyncio.client import connect
from websockets.exceptions import InvalidStatus

from aceai.llm.errors import AceAIConfigurationError, AceAIRuntimeError
from aceai.llm.models import LLMInput, LLMResponse, LLMSegment, LLMStreamEvent
from aceai.llm.openai import OpenAI, OpenAIMeta, OpenAIPayload


OPENAI_CODEX_BASE_URL = "https://chatgpt.com/backend-api/codex"
OPENAI_CODEX_REALTIME_TRANSCRIPTION_URL = "wss://chatgpt.com/backend-api/codex/realtime"
OPENAI_CODEX_PROVIDER_NAME = "codex"
OPENAI_CODEX_CLIENT_TIMEOUT_SECONDS = 300.0
OPENAI_CODEX_REALTIME_AUDIO_RATE = 24_000
OPENAI_CODEX_REALTIME_AUDIO_CHANNELS = 1
OPENAI_CODEX_REALTIME_AUDIO_SAMPLE_WIDTH = 2
OPENAI_CODEX_REALTIME_AUDIO_CHUNK_BYTES = 32_000


class OpenAICodex(OpenAI):
    """OpenAI Codex provider using a resolved ChatGPT/Codex access token."""

    def __init__(
        self,
        *,
        api_key: str,
        default_meta: OpenAIMeta,
        base_url: str = OPENAI_CODEX_BASE_URL,
        realtime_transcription_url: str = OPENAI_CODEX_REALTIME_TRANSCRIPTION_URL,
        websocket_connect: Callable[..., Any] = connect,
        instructions: str = "",
    ):
        if api_key == "":
            raise AceAIConfigurationError("OpenAI Codex access token is required")
        self._api_key = api_key
        self._realtime_transcription_url = realtime_transcription_url
        self._websocket_connect = websocket_connect
        super().__init__(
            client=AsyncOpenAI(
                api_key=api_key,
                base_url=base_url,
                timeout=OPENAI_CODEX_CLIENT_TIMEOUT_SECONDS,
            ),
            default_meta=default_meta,
            provider_name=OPENAI_CODEX_PROVIDER_NAME,
        )
        self._instructions = instructions

    async def stt(
        self,
        filename: str,
        file: BinaryIO,
        *,
        model: str,
        prompt: str | None = None,
    ) -> str:
        if prompt is not None:
            raise ValueError("Codex STT does not support transcription prompts")
        pcm = _pcm_from_wav(file.read())
        return await self._transcribe_pcm_with_realtime(
            pcm,
            model=model,
            session_id=f"aceai-stt-{filename}",
        )

    def _build_response_kwargs(self, payload: OpenAIPayload) -> dict[str, Any]:
        kwargs = super()._build_response_kwargs(payload)
        if self._instructions != "":
            kwargs["instructions"] = self._instructions
        kwargs["store"] = False
        if "max_output_tokens" in kwargs:
            del kwargs["max_output_tokens"]
        return kwargs

    async def complete(self, request: LLMInput) -> LLMResponse:
        response: LLMResponse | None = None
        async for event in self.stream(request):
            if event.event_type == "response.completed" and isinstance(
                event.response,
                LLMResponse,
            ):
                response = event.response
        if response is None:
            raise AceAIRuntimeError("OpenAI Codex stream did not complete")
        return response

    async def stream(self, request: LLMInput) -> AsyncGenerator[LLMStreamEvent, None]:
        text = ""
        async for event in super().stream(request):
            if event.event_type == "response.output_text.delta":
                if isinstance(event.text_delta, str):
                    text += event.text_delta
            elif event.event_type == "response.completed":
                response = event.response
                if (
                    isinstance(response, LLMResponse)
                    and response.text == ""
                    and text != ""
                ):
                    segments = [LLMSegment(type="text", content=text)]
                    segments.extend(response.segments)
                    patched_response = LLMResponse(
                        id=response.id,
                        model=response.model,
                        text=text,
                        tool_calls=response.tool_calls,
                        usage=response.usage,
                        segments=segments,
                        provider_meta=response.provider_meta,
                        status=response.status,
                        reasoning=response.reasoning,
                        reasoning_content=response.reasoning_content,
                    )
                    event = LLMStreamEvent(
                        event_type="response.completed",
                        response=patched_response,
                        segments=patched_response.segments,
                        provider_meta=patched_response.provider_meta,
                    )
            yield event

    async def _transcribe_pcm_with_realtime(
        self,
        pcm: bytes,
        *,
        model: str,
        session_id: str,
    ) -> str:
        headers = {
            "Authorization": f"Bearer {self._api_key}",
            "x-session-id": session_id,
        }
        try:
            websocket_context = self._websocket_connect(
                self._realtime_transcription_url,
                additional_headers=headers,
            )
            async with websocket_context as websocket:
                return await self._transcribe_pcm_with_realtime_websocket(
                    websocket,
                    pcm,
                    model=model,
                )
        except InvalidStatus as exc:
            if exc.response.status_code == 403:
                raise AceAIRuntimeError(
                    "Codex subscription STT requires the Codex realtime call flow; "
                    "direct realtime websocket transcription was rejected."
                ) from exc
            raise

    async def _transcribe_pcm_with_realtime_websocket(
        self,
        websocket: Any,
        pcm: bytes,
        *,
        model: str,
    ) -> str:
        await websocket.send(
            json.dumps(
                {
                    "type": "session.update",
                    "session": {
                        "type": "transcription",
                        "audio": {
                            "input": {
                                "format": {
                                    "type": "audio/pcm",
                                    "rate": OPENAI_CODEX_REALTIME_AUDIO_RATE,
                                },
                                "transcription": {"model": model},
                            }
                        },
                    },
                }
            )
        )
        for offset in range(0, len(pcm), OPENAI_CODEX_REALTIME_AUDIO_CHUNK_BYTES):
            chunk = pcm[offset : offset + OPENAI_CODEX_REALTIME_AUDIO_CHUNK_BYTES]
            await websocket.send(
                json.dumps(
                    {
                        "type": "input_audio_buffer.append",
                        "audio": base64.b64encode(chunk).decode("ascii"),
                    }
                )
            )
        await websocket.send(json.dumps({"type": "input_audio_buffer.commit"}))
        while True:
            raw_event = await websocket.recv()
            event = _realtime_event(raw_event)
            event_type = event["type"]
            if event_type == "conversation.item.input_audio_transcription.completed":
                transcript = event["transcript"]
                if type(transcript) is not str:
                    raise TypeError("Codex STT transcript must be str")
                return transcript
            if event_type == "error":
                error = event["error"]
                if type(error) is dict and type(error.get("message")) is str:
                    raise AceAIRuntimeError(error["message"])
                raise AceAIRuntimeError(json.dumps(event))


def _pcm_from_wav(audio: bytes) -> bytes:
    with wave.open(io.BytesIO(audio), "rb") as wav:
        if wav.getnchannels() != OPENAI_CODEX_REALTIME_AUDIO_CHANNELS:
            raise ValueError("Codex STT requires mono WAV audio")
        if wav.getframerate() != OPENAI_CODEX_REALTIME_AUDIO_RATE:
            raise ValueError("Codex STT requires 24 kHz WAV audio")
        if wav.getsampwidth() != OPENAI_CODEX_REALTIME_AUDIO_SAMPLE_WIDTH:
            raise ValueError("Codex STT requires 16-bit PCM WAV audio")
        if wav.getcomptype() != "NONE":
            raise ValueError("Codex STT requires uncompressed PCM WAV audio")
        return wav.readframes(wav.getnframes())


def _realtime_event(raw_event: str | bytes) -> dict[str, Any]:
    if type(raw_event) is bytes:
        raw_event = raw_event.decode("utf-8")
    event = json.loads(raw_event)
    if type(event) is not dict:
        raise TypeError("Codex realtime event must be an object")
    event_type = event["type"]
    if type(event_type) is not str:
        raise TypeError("Codex realtime event type must be str")
    return event



import subprocess
import textwrap
from pathlib import Path


PROJECT_ROOT = Path(__file__).resolve().parents[1]


def test_base_wheel_install_does_not_require_provider_sdks(tmp_path: Path) -> None:
    wheel = _build_wheel(tmp_path)
    python = _install_wheel(tmp_path / "base-venv", wheel)

    script = """
        import asyncio
        import importlib.util

        import aceai
        import aceai.llm
        from aceai import Agent, Executor, Graph, tool
        from aceai.llm import LLMMessage, LLMService
        from aceai.llm.models import LLMResponse, LLMStreamEvent

        assert importlib.util.find_spec("openai") is None
        assert importlib.util.find_spec("websockets") is None

        class FakeLLMService:
            async def stream(self, **request):
                yield LLMStreamEvent(
                    event_type="response.completed",
                    response=LLMResponse(text="base-ok"),
                )

        @tool
        def ping() -> str:
            return "pong"

        async def main():
            agent = Agent(
                prompt="Smoke test agent.",
                default_model="fake-model",
                llm_service=FakeLLMService(),
                executor=Executor(Graph(), [ping]),
                max_steps=1,
            )
            answer = await agent.ask("say smoke")
            assert answer == "base-ok"

        asyncio.run(main())

        try:
            import aceai.llm.openai
        except RuntimeError as exc:
            assert "aceai[openai]" in str(exc)
        else:
            raise AssertionError("OpenAI provider imported without the openai extra")

        print(aceai.__version__, Agent.__name__, Executor.__name__, LLMService.__name__, LLMMessage.__name__)
    """
    _run_python(python, script, cwd=tmp_path)


def test_provider_extras_make_provider_modules_importable(tmp_path: Path) -> None:
    wheel = _build_wheel(tmp_path)
    python = _install_wheel(tmp_path / "providers-venv", f"{wheel}[providers]")

    script = """
        from aceai.llm.anthropic import Anthropic
        from aceai.llm.deepseek import DeepSeek
        from aceai.llm.openai import OpenAI
        from aceai.llm.openai_codex import OpenAICodex

        assert Anthropic.__name__ == "Anthropic"
        assert DeepSeek.__name__ == "DeepSeek"
        assert OpenAI.__name__ == "OpenAI"
        assert OpenAICodex.__name__ == "OpenAICodex"
    """
    _run_python(python, script, cwd=tmp_path)


def _build_wheel(tmp_path: Path) -> Path:
    dist_dir = tmp_path / "dist"
    _run(["uv", "build", "--wheel", "--out-dir", str(dist_dir)], cwd=PROJECT_ROOT)
    wheels = sorted(dist_dir.glob("*.whl"))
    assert len(wheels) == 1
    return wheels[0]


def _install_wheel(venv_dir: Path, wheel: Path | str) -> Path:
    _run(["uv", "venv", str(venv_dir)])
    python = venv_dir / "bin" / "python"
    _run(["uv", "pip", "install", "--python", str(python), str(wheel)])
    return python


def _run_python(python: Path, script: str, *, cwd: Path) -> None:
    _run([str(python), "-c", textwrap.dedent(script)], cwd=cwd)


def _run(command: list[str], *, cwd: Path | None = None) -> None:
    subprocess.run(
        command,
        cwd=cwd,
        check=True,
        text=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )



from pathlib import Path
from typing import AsyncGenerator

import httpx
import pytest
from ididi import Graph, use
from msgspec import Struct

from aceai.llm.errors import AceAIRuntimeError
from aceai.core.executor import (
    DummyExecutor,
    Executor,
    LoggingExecutor,
    ToolExecutionError,
)
from aceai.core.output_truncation import DEFAULT_TRUNCATED_OUTPUT_TOKEN_BUDGET
from aceai.core.run_state import ToolRunState
from aceai.llm.models import LLMToolCall
from aceai.core.tools import tool
from aceai.core.tools._tool_sig import Annotated, spec


class FakeLogger:
    def __init__(self):
        self.info_messages: list[str] = []
        self.success_messages: list[str] = []
        self.exception_messages: list[str] = []

    def info(self, msg: str, /, **_: object) -> None:
        self.info_messages.append(msg)

    def success(self, msg: str, /, **_: object) -> None:
        self.success_messages.append(msg)

    def exception(self, msg: str, /, **_: object) -> None:
        self.exception_messages.append(msg)


class StepTimer:
    def __init__(self, *ticks: float):
        self._ticks = iter(ticks)

    def __call__(self) -> float:
        return next(self._ticks)


class UserRepo:
    def __init__(self, label: str):
        self.label = label


def provide_user_repo() -> UserRepo:
    return UserRepo("primary")


def describe_user(
    repo: Annotated[UserRepo, use(provide_user_repo)],
    user_id: Annotated[int, spec(description="User identifier")],
) -> str:
    return f"{repo.label}:{user_id}"


async def async_increment(
    value: Annotated[int, spec(description="Value to increment")],
) -> int:
    return value + 1


def unreliable_tool(
    value: Annotated[int, spec(description="Value that triggers failure")],
) -> int:
    raise AceAIRuntimeError("expected failure")


def echo_message(
    message: Annotated[str, spec(description="Message to echo")],
) -> str:
    return message


def long_message() -> str:
    return "tool-output-line\n" * 5000


def build_async_client() -> httpx.AsyncClient:
    return httpx.AsyncClient()


async def identify_httpx_client(
    client: Annotated[httpx.AsyncClient, use(build_async_client)],
) -> str:
    try:
        return client.__class__.__name__
    finally:
        await client.aclose()


def build_tool(func):
    return tool(func)


def write_skill(root: Path, name: str, description: str, body: str) -> Path:
    skill_dir = root / name
    skill_dir.mkdir(parents=True)
    (skill_dir / "SKILL.md").write_text(
        "\n".join(
            [
                "---",
                f"name: {name}",
                f"description: {description}",
                "---",
                body,
            ]
        ),
        encoding="utf-8",
    )
    return skill_dir


async def execute_call(
    executor: Executor,
    call: LLMToolCall,
    tool_state: ToolRunState | None = None,
) -> str:
    result = await executor.execute(
        executor.resolve_invocation(call),
        tool_state=tool_state or ToolRunState(),
    )
    assert result.truncated_output == result.output
    return result.output


@pytest.mark.anyio
async def test_tool_executor_truncates_default_truncated_output(graph: Graph) -> None:
    executor = Executor(graph, [tool(long_message)])
    call = LLMToolCall(
        name="long_message",
        arguments="{}",
        call_id="call-long-message",
    )

    result = await executor.execute(
        executor.resolve_invocation(call),
        tool_state=ToolRunState(),
    )

    assert result.output.startswith('"tool-output-line\\n')
    assert result.truncated_output != result.output
    assert "tokens truncated" in result.truncated_output
    assert len(result.truncated_output) < DEFAULT_TRUNCATED_OUTPUT_TOKEN_BUDGET * 5


def test_dummy_executor_has_no_tools() -> None:
    executor = DummyExecutor()

    assert executor.select_tools() == []
    assert executor.select_tools(include={"missing"}) == []
    assert executor.select_tools(exclude={"missing"}) == []


def test_dummy_executor_rejects_tool_invocation() -> None:
    executor = DummyExecutor()
    call = LLMToolCall(name="missing", arguments="{}", call_id="call-missing")

    with pytest.raises(KeyError, match="missing"):
        executor.resolve_invocation(call)


def test_executor_loads_skill_registry_and_prompt_instructions(
    tmp_path: Path, monkeypatch: pytest.MonkeyPatch
) -> None:
    home = tmp_path / "home"
    cwd = tmp_path / "project"
    write_skill(home / ".aceai" / "skills", "release", "Release workflow.", "# Release")
    write_skill(cwd / ".agents" / "skills", "review", "Review workflow.", "# Review")
    monkeypatch.setenv("HOME", str(home))
    monkeypatch.chdir(cwd)

    executor = Executor(Graph(), [], skill_path="auto")

    assert set(executor.skill_registry.skills) == {"release", "review"}
    assert "<available_skills>" in executor.prompt_instructions
    assert "<name>release</name>" in executor.prompt_instructions
    assert "<name>review</name>" in executor.prompt_instructions
    assert "skills_list" in executor.tools
    assert "skill_view" in executor.tools


def test_executor_loads_extra_skill_paths_after_user_paths(
    tmp_path: Path, monkeypatch: pytest.MonkeyPatch
) -> None:
    home = tmp_path / "home"
    cwd = tmp_path / "project"
    extra = tmp_path / "builtin"
    write_skill(home / ".aceai" / "skills", "release", "Release workflow.", "# Release")
    write_skill(cwd / ".agents" / "skills", "review", "Review workflow.", "# Review")
    write_skill(extra, "skill-creator", "Create skills.", "# Skill Creator")
    monkeypatch.setenv("HOME", str(home))
    monkeypatch.chdir(cwd)

    executor = Executor(
        Graph(),
        [],
        skill_path="auto",
        extra_skill_paths=(extra,),
    )

    assert set(executor.skill_registry.skills) == {
        "release",
        "review",
        "skill-creator",
    }
    assert "<name>skill-creator</name>" in executor.prompt_instructions


def test_executor_extra_skill_paths_do_not_override_user_skills(
    tmp_path: Path, monkeypatch: pytest.MonkeyPatch
) -> None:
    home = tmp_path / "home"
    cwd = tmp_path / "project"
    extra = tmp_path / "builtin"
    write_skill(
        home / ".aceai" / "skills",
        "skill-creator",
        "User skill creator.",
        "# User Skill Creator",
    )
    write_skill(extra, "skill-creator", "Builtin skill creator.", "# Builtin")
    monkeypatch.setenv("HOME", str(home))
    cwd.mkdir()
    monkeypatch.chdir(cwd)

    executor = Executor(
        Graph(),
        [],
        skill_path="auto",
        extra_skill_paths=(extra,),
    )

    skill = executor.skill_registry.get("skill-creator")
    assert skill.description == "User skill creator."
    assert skill.read_instructions() == "# User Skill Creator"


def test_executor_disable_skill_path_skips_extra_skill_paths(tmp_path: Path) -> None:
    extra = tmp_path / "builtin"
    write_skill(extra, "skill-creator", "Create skills.", "# Skill Creator")

    executor = Executor(
        Graph(),
        [],
        skill_path="disable",
        extra_skill_paths=(extra,),
    )

    assert executor.skill_registry.skills == {}
    assert "skills_list" not in executor.tools


def test_executor_filters_enabled_skills(
    tmp_path: Path, monkeypatch: pytest.MonkeyPatch
) -> None:
    home = tmp_path / "home"
    cwd = tmp_path / "project"
    write_skill(home / ".aceai" / "skills", "release", "Release workflow.", "# Release")
    write_skill(cwd / ".agents" / "skills", "review", "Review workflow.", "# Review")
    monkeypatch.setenv("HOME", str(home))
    monkeypatch.chdir(cwd)

    executor = Executor(
        Graph(),
        [],
        skill_path="auto",
        enabled_skill_names=("review",),
    )

    assert set(executor.skill_registry.skills) == {"review"}
    assert "<name>review</name>" in executor.prompt_instructions
    assert "<name>release</name>" not in executor.prompt_instructions


@pytest.fixture
async def graph() -> AsyncGenerator[Graph, None]:
    graph = Graph()
    try:
        yield graph
    finally:
        graph._workers.shutdown(wait=True)


@pytest.mark.anyio
async def test_tool_executor_executes_tool_with_dep_graph(graph: Graph) -> None:
    describe_tool = build_tool(describe_user)
    executor = Executor(graph, [describe_tool])

    call = LLMToolCall(
        name=describe_tool.name,
        arguments='{"user_id":7}',
        call_id="call-123",
    )

    encoded_result = await execute_call(executor, call)

    assert encoded_result == '"primary:7"'


@pytest.mark.anyio
async def test_tool_executor_awaits_async_tool_results(graph: Graph) -> None:
    increment_tool = build_tool(async_increment)
    executor = Executor(graph, [increment_tool])

    call = LLMToolCall(
        name=increment_tool.name,
        arguments='{"value":2}',
        call_id="call-async",
    )

    result = await execute_call(executor, call)

    assert result == "3"


@pytest.mark.anyio
async def test_tool_executor_returns_invalid_arguments_as_tool_execution_error(
    graph: Graph,
) -> None:
    message_tool = build_tool(echo_message)
    executor = Executor(graph, [message_tool])
    call = LLMToolCall(
        name=message_tool.name,
        arguments='{"query":"hello"}',
        call_id="call-invalid-args",
    )

    with pytest.raises(ToolExecutionError) as exc_info:
        await execute_call(executor, call)

    assert str(exc_info.value) == (
        "Invalid arguments for tool echo_message: Object missing required field `message`"
    )


@pytest.mark.anyio
async def test_logging_tool_executor_logs_successful_calls(graph: Graph) -> None:
    increment_tool = build_tool(async_increment)
    logger = FakeLogger()
    timer = StepTimer(10.0, 10.5)
    executor = LoggingExecutor(
        graph=graph,
        tools=[increment_tool],
        logger=logger,
        timer=timer,
    )

    call = LLMToolCall(
        name=increment_tool.name,
        arguments='{"value":2}',
        call_id="req-success",
    )

    result = await execute_call(executor, call)

    assert result == "3"
    assert logger.info_messages == [
        'Tool async_increment starting (call_id=req-success) with {"value":2}'
    ]
    assert logger.success_messages == [
        "Tool async_increment finished in 0.50s, result: 3"
    ]
    assert logger.exception_messages == []


@pytest.mark.anyio
async def test_logging_tool_executor_logs_and_reraises_failures(graph: Graph) -> None:
    failing_tool = build_tool(unreliable_tool)
    logger = FakeLogger()
    timer = StepTimer(5.0, 6.25)
    executor = LoggingExecutor(
        graph=graph,
        tools=[failing_tool],
        logger=logger,
        timer=timer,
    )

    call = LLMToolCall(
        name=failing_tool.name,
        arguments='{"value":1}',
        call_id="req-fail",
    )

    with pytest.raises(AceAIRuntimeError, match="expected failure"):
        await execute_call(executor, call)

    assert logger.info_messages == [
        'Tool unreliable_tool starting (call_id=req-fail) with {"value":1}'
    ]
    assert logger.exception_messages == ["Tool unreliable_tool failed after 1.25s"]
    assert logger.success_messages == []


@pytest.mark.anyio
async def test_tool_executor_resolves_httpx_async_client(graph: Graph) -> None:
    client_tool = build_tool(identify_httpx_client)
    executor = Executor(graph, [client_tool])

    call = LLMToolCall(name=client_tool.name, arguments="{}", call_id="req-httpx")

    result = await execute_call(executor, call)

    assert result == '"AsyncClient"'


class Payload(Struct):
    value: int


def build_struct(value: Annotated[int, spec(description="Value to wrap")]) -> Payload:
    return Payload(value)


@pytest.mark.anyio
async def test_tool_executor_encodes_struct_return(graph: Graph) -> None:
    struct_tool = build_tool(build_struct)
    executor = Executor(graph, [struct_tool])

    call = LLMToolCall(
        name=struct_tool.name,
        arguments='{"value":5}',
        call_id="req-struct",
    )

    encoded = await execute_call(executor, call)

    assert encoded == '{"value":5}'


@pytest.mark.anyio
async def test_tool_executor_exposes_tool_specs(graph: Graph) -> None:
    echo_tool = build_tool(echo_message)
    executor = Executor(graph, [echo_tool])

    first = executor.all_tools
    second = executor.all_tools

    assert first is second
    schema_names = {spec.name for spec in first}
    assert echo_tool.name in schema_names


@pytest.mark.anyio
async def test_tool_executor_enforces_max_calls_per_run(graph: Graph) -> None:
    executed: list[int] = []

    def tick() -> int:
        executed.append(1)
        return len(executed)

    tick_tool = tool(max_calls_per_run=1)(tick)
    executor = Executor(graph, [tick_tool])
    tool_state = ToolRunState()

    first = await execute_call(
        executor,
        LLMToolCall(name=tick_tool.name, arguments="{}", call_id="call-1"),
        tool_state,
    )
    second = await execute_call(
        executor,
        LLMToolCall(name=tick_tool.name, arguments="{}", call_id="call-2"),
        tool_state,
    )

    assert first == "1"
    assert second == (
        "the tool tick exceeds its max calls in this run, do not call it again"
    )
    assert executed == [1]


@pytest.mark.anyio
async def test_tool_executor_resolves_invocation_with_approval_metadata(
    graph: Graph,
) -> None:
    approved_tool = tool(require_approval=True)(echo_message)
    executor = Executor(graph, [approved_tool])
    call = LLMToolCall(
        name=approved_tool.name,
        arguments='{"message":"hello"}',
        call_id="call-approval",
    )

    invocation = executor.resolve_invocation(call)
    result = await executor.execute(invocation, tool_state=ToolRunState())

    assert invocation.call is call
    assert invocation.tool is approved_tool
    assert invocation.approval_required is True
    assert result.output == '"hello"'
    assert result.truncated_output == '"hello"'
